@@ -4,6 +4,7 @@ use std::fmt;
 use std::marker::PhantomData;
 use std::mem;
 use std::ops;
+use std::ops::ControlFlow;
 use std::ptr;
 use std::slice;
 use std::u64;
@@ -22,6 +23,12 @@ use glib::translate::{from_glib, from_glib_full, FromGlib, FromGlibPtrFull, Into
 pub enum Readable {}
 pub enum Writable {}
 
+#[derive(Copy, Clone, Debug, PartialEq, Eq)]
+pub enum BufferMetaForeachAction {
+    Keep,
+    Remove,
+}
+
 mini_object_wrapper!(Buffer, BufferRef, ffi::GstBuffer, || {
     ffi::gst_buffer_get_type()
 });
@@ -46,6 +53,8 @@ impl Buffer {
         unsafe { from_glib_full(ffi::gst_buffer_new()) }
     }
 
+    #[doc(alias = "gst_buffer_new_allocate")]
+    #[doc(alias = "gst_buffer_new_and_alloc")]
     pub fn with_size(size: usize) -> Result<Self, glib::BoolError> {
         assert_initialized_main_thread!();
 
@@ -64,6 +73,7 @@ impl Buffer {
         drop(slice);
     }
 
+    #[doc(alias = "gst_buffer_new_wrapped_full")]
     pub fn from_mut_slice<T: AsMut<[u8]> + Send + 'static>(slice: T) -> Self {
         assert_initialized_main_thread!();
 
@@ -86,6 +96,7 @@ impl Buffer {
         }
     }
 
+    #[doc(alias = "gst_buffer_new_wrapped_full")]
     pub fn from_slice<T: AsRef<[u8]> + Send + 'static>(slice: T) -> Self {
         assert_initialized_main_thread!();
 
@@ -108,6 +119,7 @@ impl Buffer {
         }
     }
 
+    #[doc(alias = "gst_buffer_map")]
     pub fn into_mapped_buffer_readable(self) -> Result<MappedBuffer<Readable>, Self> {
         unsafe {
             let mut map_info = mem::MaybeUninit::zeroed();
@@ -128,6 +140,7 @@ impl Buffer {
         }
     }
 
+    #[doc(alias = "gst_buffer_map")]
     pub fn into_mapped_buffer_writable(self) -> Result<MappedBuffer<Writable>, Self> {
         unsafe {
             let mut map_info = mem::MaybeUninit::zeroed();
@@ -173,6 +186,7 @@ impl Default for Buffer {
 }
 
 impl BufferRef {
+    #[doc(alias = "gst_buffer_map")]
     pub fn map_readable(&self) -> Result<BufferMap<Readable>, glib::BoolError> {
         unsafe {
             let mut map_info = mem::MaybeUninit::zeroed();
@@ -190,6 +204,7 @@ impl BufferRef {
         }
     }
 
+    #[doc(alias = "gst_buffer_map")]
     pub fn map_writable(&mut self) -> Result<BufferMap<Writable>, glib::BoolError> {
         unsafe {
             let mut map_info = mem::MaybeUninit::zeroed();
@@ -252,6 +267,7 @@ impl BufferRef {
         }
     }
 
+    #[doc(alias = "gst_buffer_fill")]
     pub fn copy_from_slice(&mut self, offset: usize, slice: &[u8]) -> Result<(), usize> {
         let maxsize = self.maxsize();
         let size = slice.len();
@@ -275,6 +291,7 @@ impl BufferRef {
         }
     }
 
+    #[doc(alias = "gst_buffer_extract")]
     pub fn copy_to_slice(&self, offset: usize, slice: &mut [u8]) -> Result<(), usize> {
         let maxsize = self.size();
         let size = slice.len();
@@ -333,6 +350,7 @@ impl BufferRef {
     }
 
     #[doc(alias = "get_offset")]
+    #[doc(alias = "GST_BUFFER_OFFSET")]
     pub fn offset(&self) -> u64 {
         self.0.offset
     }
@@ -342,6 +360,7 @@ impl BufferRef {
     }
 
     #[doc(alias = "get_offset_end")]
+    #[doc(alias = "GST_BUFFER_OFFSET_END")]
     pub fn offset_end(&self) -> u64 {
         self.0.offset_end
     }
@@ -351,6 +370,7 @@ impl BufferRef {
     }
 
     #[doc(alias = "get_pts")]
+    #[doc(alias = "GST_BUFFER_PTS")]
     pub fn pts(&self) -> Option<ClockTime> {
         unsafe { from_glib(self.0.pts) }
     }
@@ -360,6 +380,7 @@ impl BufferRef {
     }
 
     #[doc(alias = "get_dts")]
+    #[doc(alias = "GST_BUFFER_DTS")]
     pub fn dts(&self) -> Option<ClockTime> {
         unsafe { from_glib(self.0.dts) }
     }
@@ -369,6 +390,7 @@ impl BufferRef {
     }
 
     #[doc(alias = "get_dts_or_pts")]
+    #[doc(alias = "GST_BUFFER_DTS_OR_PTS")]
     pub fn dts_or_pts(&self) -> Option<ClockTime> {
         let val = self.dts();
         if val.is_none() {
@@ -379,6 +401,7 @@ impl BufferRef {
     }
 
     #[doc(alias = "get_duration")]
+    #[doc(alias = "GST_BUFFER_DURATION")]
     pub fn duration(&self) -> Option<ClockTime> {
         unsafe { from_glib(self.0.duration) }
     }
@@ -388,14 +411,17 @@ impl BufferRef {
     }
 
     #[doc(alias = "get_flags")]
+    #[doc(alias = "GST_BUFFER_FLAGS")]
     pub fn flags(&self) -> BufferFlags {
         BufferFlags::from_bits_truncate(self.0.mini_object.flags)
     }
 
+    #[doc(alias = "GST_BUFFER_FLAG_SET")]
     pub fn set_flags(&mut self, flags: BufferFlags) {
         self.0.mini_object.flags |= flags.bits();
     }
 
+    #[doc(alias = "GST_BUFFER_FLAG_UNSET")]
     pub fn unset_flags(&mut self, flags: BufferFlags) {
         self.0.mini_object.flags &= !flags.bits();
     }
@@ -434,8 +460,8 @@ impl BufferRef {
     }
 
     #[doc(alias = "gst_buffer_foreach_meta")]
-    pub fn foreach_meta<F: FnMut(MetaRef<Meta>) -> bool>(&self, func: F) -> bool {
-        unsafe extern "C" fn trampoline<F: FnMut(MetaRef<Meta>) -> bool>(
+    pub fn foreach_meta<F: FnMut(MetaRef<Meta>) -> ControlFlow<(), ()>>(&self, func: F) -> bool {
+        unsafe extern "C" fn trampoline<F: FnMut(MetaRef<Meta>) -> ControlFlow<(), ()>>(
             buffer: *mut ffi::GstBuffer,
             meta: *mut *mut ffi::GstMeta,
             user_data: glib::ffi::gpointer,
@@ -443,7 +469,7 @@ impl BufferRef {
             let func = user_data as *const _ as usize as *mut F;
             let res = (*func)(Meta::from_ptr(BufferRef::from_ptr(buffer), *meta));
 
-            res.into_glib()
+            matches!(res, ControlFlow::Continue(_)).into_glib()
         }
 
         unsafe {
@@ -457,14 +483,19 @@ impl BufferRef {
         }
     }
 
+    #[doc(alias = "gst_buffer_foreach_meta")]
     pub fn foreach_meta_mut<
-        F: FnMut(MetaRefMut<Meta, crate::meta::Iterated>) -> Result<bool, bool>,
+        F: FnMut(
+            MetaRefMut<Meta, crate::meta::Iterated>,
+        ) -> ControlFlow<BufferMetaForeachAction, BufferMetaForeachAction>,
     >(
         &mut self,
         func: F,
     ) -> bool {
         unsafe extern "C" fn trampoline<
-            F: FnMut(MetaRefMut<Meta, crate::meta::Iterated>) -> Result<bool, bool>,
+            F: FnMut(
+                MetaRefMut<Meta, crate::meta::Iterated>,
+            ) -> ControlFlow<BufferMetaForeachAction, BufferMetaForeachAction>,
         >(
             buffer: *mut ffi::GstBuffer,
             meta: *mut *mut ffi::GstMeta,
@@ -473,13 +504,16 @@ impl BufferRef {
             let func = user_data as *const _ as usize as *mut F;
             let res = (*func)(Meta::from_mut_ptr(BufferRef::from_mut_ptr(buffer), *meta));
 
-            match res {
-                Ok(false) | Err(false) => {
-                    *meta = ptr::null_mut();
-                    res.is_ok().into_glib()
-                }
-                Ok(true) | Err(true) => res.is_ok().into_glib(),
+            let (cont, action) = match res {
+                ControlFlow::Continue(action) => (true, action),
+                ControlFlow::Break(action) => (false, action),
+            };
+
+            if action == BufferMetaForeachAction::Remove {
+                *meta = ptr::null_mut();
             }
+
+            cont.into_glib()
         }
 
         unsafe {
@@ -623,6 +657,7 @@ impl BufferRef {
         unsafe { MemoryRef::from_ptr(ffi::gst_buffer_peek_memory(self.as_mut_ptr(), idx)) }
     }
 
+    #[doc(alias = "gst_buffer_peek_memory")]
     pub fn peek_memory_mut(&mut self, idx: u32) -> Result<&mut MemoryRef, glib::BoolError> {
         assert!(idx < self.n_memory());
         unsafe {
@@ -903,6 +938,38 @@ define_iter!(
     }
 );
 
+impl<'a> IntoIterator for &'a BufferRef {
+    type IntoIter = Iter<'a>;
+    type Item = &'a MemoryRef;
+
+    fn into_iter(self) -> Self::IntoIter {
+        self.iter_memories()
+    }
+}
+
+impl std::iter::FromIterator<Memory> for Buffer {
+    fn from_iter<T: IntoIterator<Item = Memory>>(iter: T) -> Self {
+        assert_initialized_main_thread!();
+
+        let iter = iter.into_iter();
+
+        let mut buffer = Buffer::new();
+
+        {
+            let buffer = buffer.get_mut().unwrap();
+            iter.for_each(|m| buffer.append_memory(m));
+        }
+
+        buffer
+    }
+}
+
+impl std::iter::Extend<Memory> for BufferRef {
+    fn extend<T: IntoIterator<Item = Memory>>(&mut self, iter: T) {
+        iter.into_iter().for_each(|m| self.append_memory(m));
+    }
+}
+
 define_iter!(
     IterOwned,
     &'a BufferRef,
@@ -941,9 +1008,9 @@ impl fmt::Debug for BufferRef {
 
         f.debug_struct("Buffer")
             .field("ptr", unsafe { &self.as_ptr() })
-            .field("pts", &self.pts().display().to_string())
-            .field("dts", &self.dts().display().to_string())
-            .field("duration", &self.duration().display().to_string())
+            .field("pts", &self.pts().display())
+            .field("dts", &self.dts().display())
+            .field("duration", &self.duration().display())
             .field("size", &self.size())
             .field("offset", &self.offset())
             .field("offset_end", &self.offset_end())
@@ -1133,8 +1200,10 @@ impl<T> Eq for MappedBuffer<T> {}
 unsafe impl<T> Send for MappedBuffer<T> {}
 unsafe impl<T> Sync for MappedBuffer<T> {}
 
+#[doc(alias = "GST_BUFFER_COPY_METADATA")]
 pub const BUFFER_COPY_METADATA: crate::BufferCopyFlags =
     crate::BufferCopyFlags::from_bits_truncate(ffi::GST_BUFFER_COPY_METADATA);
+#[doc(alias = "GST_BUFFER_COPY_ALL")]
 pub const BUFFER_COPY_ALL: crate::BufferCopyFlags =
     crate::BufferCopyFlags::from_bits_truncate(ffi::GST_BUFFER_COPY_ALL);
 
@@ -1349,7 +1418,7 @@ mod tests {
                 .downcast_ref::<crate::ReferenceTimestampMeta>()
                 .unwrap();
             res.push(meta.timestamp());
-            true
+            ControlFlow::Continue(())
         });
 
         assert_eq!(&[ClockTime::ZERO, ClockTime::SECOND][..], &res[..]);
@@ -1385,9 +1454,9 @@ mod tests {
                 .unwrap();
             res.push(meta.timestamp());
             if meta.timestamp() == ClockTime::SECOND {
-                Ok(false)
+                ControlFlow::Continue(BufferMetaForeachAction::Remove)
             } else {
-                Ok(true)
+                ControlFlow::Continue(BufferMetaForeachAction::Keep)
             }
         });
 
@@ -1399,7 +1468,7 @@ mod tests {
                 .downcast_ref::<crate::ReferenceTimestampMeta>()
                 .unwrap();
             res.push(meta.timestamp());
-            true
+            ControlFlow::Continue(())
         });
 
         assert_eq!(&[ClockTime::ZERO][..], &res[..]);
