@@ -2,6 +2,10 @@
 
 #![cfg_attr(feature = "dox", feature(doc_cfg))]
 #![recursion_limit = "256"]
+#![allow(clippy::missing_safety_doc)]
+#![allow(clippy::manual_range_contains)]
+#![allow(clippy::non_send_fields_in_send_ty)]
+#![doc = include_str!("../README.md")]
 
 // Re-exported for the subclass gst_plugin_define! macro
 pub use ffi;
@@ -28,10 +32,14 @@ macro_rules! skip_assert_initialized {
 #[allow(clippy::type_complexity)]
 #[allow(clippy::use_self)]
 #[allow(clippy::needless_borrow)]
+#[allow(unused_imports)]
 mod auto;
 pub use crate::auto::functions::*;
 pub use crate::auto::*;
 
+#[macro_use]
+mod macros;
+
 #[macro_use]
 mod log;
 pub use crate::log::*;
@@ -42,6 +50,7 @@ pub use crate::error::*;
 
 #[macro_use]
 pub mod miniobject;
+pub use miniobject::{MiniObject, MiniObjectRef};
 pub mod message;
 pub use crate::message::{Message, MessageErrorDomain, MessageRef, MessageView};
 
@@ -57,7 +66,7 @@ pub use crate::structure::{Structure, StructureRef};
 mod structure_serde;
 
 pub mod caps;
-pub use crate::caps::{Caps, CapsRef};
+pub use crate::caps::{Caps, CapsFilterMapAction, CapsRef};
 mod caps_features;
 #[cfg(feature = "ser_de")]
 mod caps_serde;
@@ -125,6 +134,7 @@ pub use promise::{Promise, PromiseError};
 
 pub mod bus;
 mod element;
+mod element_factory;
 
 mod bin;
 
@@ -133,26 +143,21 @@ mod pipeline;
 mod allocation_params;
 pub use self::allocation_params::AllocationParams;
 
-mod element_factory_list_type;
-pub use element_factory_list_type::*;
+mod element_factory_type;
+pub use element_factory_type::*;
 
-// OS dependent Bus extensions (also import the other plateform mod for doc)
-#[cfg(any(feature = "v1_14", feature = "dox"))]
+mod tracer;
+mod tracer_factory;
+
+// OS dependent Bus extensions (also import the other platform mod for doc)
+#[cfg(any(all(unix, feature = "v1_14"), feature = "dox"))]
 #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_14")))]
-cfg_if::cfg_if! {
-    if #[cfg(unix)] {
-        mod bus_unix;
-        #[cfg(feature = "dox")]
-        mod bus_windows;
-    } else {
-        mod bus_windows;
-        #[cfg(feature = "dox")]
-        mod bus_unix;
-    }
-}
+mod bus_unix;
+#[cfg(any(all(windows, feature = "v1_14"), feature = "dox"))]
+#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_14")))]
+mod bus_windows;
 
 mod child_proxy;
-#[macro_use]
 mod clock_time;
 #[cfg(feature = "ser_de")]
 mod clock_time_serde;
@@ -161,6 +166,7 @@ mod date_time;
 mod date_time_serde;
 mod device_monitor;
 mod device_provider;
+mod device_provider_factory;
 mod enums;
 pub use crate::enums::MessageType;
 mod ghost_pad;
@@ -168,6 +174,8 @@ mod gobject;
 mod iterator;
 mod object;
 mod pad;
+pub use pad::*;
+mod registry;
 pub use crate::pad::PadBuilder;
 mod control_binding;
 mod control_source;
@@ -201,6 +209,7 @@ pub mod stream_collection;
 
 mod typefind;
 pub use crate::typefind::*;
+mod typefind_factory;
 
 pub mod format;
 pub use crate::format::{
@@ -279,25 +288,19 @@ pub const PARAM_FLAG_CONDITIONALLY_AVAILABLE: glib::ParamFlags = glib::ParamFlag
 pub mod prelude {
     #[doc(hidden)]
     pub use glib::prelude::*;
+    pub use opt_ops::prelude::*;
 
     pub use crate::auto::traits::*;
 
     pub use crate::meta::MetaAPI;
 
     // OS dependent Bus extensions (also import the other platform trait for doc)
-    #[cfg(any(feature = "v1_14", feature = "dox"))]
+    #[cfg(any(all(unix, feature = "v1_14"), feature = "dox"))]
     #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_14")))]
-    cfg_if::cfg_if! {
-        if #[cfg(unix)] {
-            pub use crate::bus_unix::UnixBusExtManual;
-            #[cfg(feature = "dox")]
-            pub use crate::bus_windows::WindowsBusExtManual;
-        } else {
-            pub use crate::bus_windows::WindowsBusExtManual;
-            #[cfg(feature = "dox")]
-            pub use crate::bus_unix::UnixBusExtManual;
-        }
-    }
+    pub use crate::bus_unix::UnixBusExtManual;
+    #[cfg(any(all(windows, feature = "v1_14"), feature = "dox"))]
+    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_14")))]
+    pub use crate::bus_windows::WindowsBusExtManual;
 
     pub use crate::bin::GstBinExtManual;
     pub use crate::buffer_pool::BufferPoolExtManual;
@@ -310,7 +313,6 @@ pub mod prelude {
     pub use crate::message::MessageErrorDomain;
     pub use crate::object::GstObjectExtManual;
     pub use crate::pad::PadExtManual;
-    pub use crate::param_spec::GstParamSpecExt;
     pub use crate::pipeline::GstPipelineExtManual;
     pub use crate::plugin::GstPluginExtManual;
     pub use crate::plugin_feature::PluginFeatureExtManual;
@@ -319,6 +321,8 @@ pub mod prelude {
     pub use crate::typefind::TypeFindImpl;
     pub use crate::value::GstValueExt;
 
+    pub use crate::miniobject::IsMiniObject;
+
     pub use crate::tags::{CustomTag, Tag};
 
     pub use muldiv::MulDiv;
