@@ -44,6 +44,9 @@ impl DebugMessage {
 pub struct DebugCategory(Option<ptr::NonNull<ffi::GstDebugCategory>>);
 
 impl DebugCategory {
+    #[doc(alias = "gst_debug_category_new")]
+    #[doc(alias = "GST_DEBUG_CATEGORY")]
+    #[doc(alias = "GST_DEBUG_CATEGORY_INIT")]
     pub fn new(
         name: &str,
         color: crate::DebugColorFlags,
@@ -70,6 +73,7 @@ impl DebugCategory {
         }
     }
 
+    #[doc(alias = "gst_debug_get_category")]
     pub fn get(name: &str) -> Option<DebugCategory> {
         skip_assert_initialized!();
         unsafe {
@@ -151,6 +155,7 @@ impl DebugCategory {
 
     #[inline]
     #[doc(alias = "gst_debug_log")]
+    #[doc(alias = "gst_debug_log_literal")]
     pub fn log<O: IsA<glib::Object>>(
         self,
         obj: Option<&O>,
@@ -176,17 +181,77 @@ impl DebugCategory {
             None => ptr::null_mut(),
         };
 
-        unsafe {
-            ffi::gst_debug_log(
-                cat.as_ptr(),
-                level.into_glib(),
-                file.to_glib_none().0,
-                module.to_glib_none().0,
-                line as i32,
-                obj_ptr,
-                fmt::format(args).replace("%", "%%").to_glib_none().0,
-            );
+        #[cfg(feature = "v1_20")]
+        {
+            let mut w = glib::GStringBuilder::default();
+
+            // Can't really happen but better safe than sorry
+            if fmt::write(&mut w, args).is_err() {
+                return;
+            }
+
+            unsafe {
+                ffi::gst_debug_log_literal(
+                    cat.as_ptr(),
+                    level.into_glib(),
+                    file.to_glib_none().0,
+                    module.to_glib_none().0,
+                    line as i32,
+                    obj_ptr,
+                    w.into_string().to_glib_none().0,
+                );
+            }
         }
+        #[cfg(not(feature = "v1_20"))]
+        {
+            // Replace literal percentage signs with two so that they are not interpreted as printf
+            // format specifiers
+            struct Write(glib::GStringBuilder);
+            impl fmt::Write for Write {
+                fn write_str(&mut self, mut s: &str) -> Result<(), fmt::Error> {
+                    while let Some((prefix, suffix)) = s.split_once('%') {
+                        self.0.append(prefix);
+                        self.0.append("%%");
+                        s = suffix;
+                    }
+                    self.0.append(s);
+                    Ok(())
+                }
+
+                fn write_char(&mut self, c: char) -> fmt::Result {
+                    if c == '%' {
+                        self.0.append("%%");
+                    } else {
+                        self.0.append_c(c);
+                    }
+                    Ok(())
+                }
+            }
+            let mut w = Write(glib::GStringBuilder::default());
+
+            // Can't really happen but better safe than sorry
+            if fmt::write(&mut w, args).is_err() {
+                return;
+            }
+
+            unsafe {
+                ffi::gst_debug_log(
+                    cat.as_ptr(),
+                    level.into_glib(),
+                    file.to_glib_none().0,
+                    module.to_glib_none().0,
+                    line as i32,
+                    obj_ptr,
+                    w.0.into_string().to_glib_none().0,
+                );
+            }
+        }
+    }
+
+    #[doc(alias = "get_all_categories")]
+    #[doc(alias = "gst_debug_get_all_categories")]
+    pub fn all_categories() -> glib::SList<DebugCategory> {
+        unsafe { glib::SList::from_glib_container_static(ffi::gst_debug_get_all_categories()) }
     }
 }
 
@@ -199,6 +264,24 @@ impl fmt::Debug for DebugCategory {
     }
 }
 
+impl GlibPtrDefault for DebugCategory {
+    type GlibType = *mut ffi::GstDebugCategory;
+}
+
+impl FromGlibPtrNone<*mut ffi::GstDebugCategory> for DebugCategory {
+    unsafe fn from_glib_none(ptr: *mut ffi::GstDebugCategory) -> Self {
+        assert!(!ptr.is_null());
+        DebugCategory(Some(ptr::NonNull::new_unchecked(ptr)))
+    }
+}
+
+impl FromGlibPtrFull<*mut ffi::GstDebugCategory> for DebugCategory {
+    unsafe fn from_glib_full(ptr: *mut ffi::GstDebugCategory) -> Self {
+        assert!(!ptr.is_null());
+        DebugCategory(Some(ptr::NonNull::new_unchecked(ptr)))
+    }
+}
+
 pub static CAT_RUST: Lazy<DebugCategory> = Lazy::new(|| {
     DebugCategory::new(
         "GST_RUST",
@@ -519,6 +602,13 @@ mod tests {
         assert_eq!(perf_cat.name(), CAT_PERFORMANCE.name());
     }
 
+    #[test]
+    fn all() {
+        crate::init().unwrap();
+
+        assert!(DebugCategory::all_categories().any(|c| c.name() == "GST_PERFORMANCE"));
+    }
+
     #[test]
     fn new_and_log() {
         crate::init().unwrap();
